%
%
%  Split Spot Function for Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="Spots Functions">
%        <Item name="Split Spots" icon="Matlab" tooltip="Split one spots object in sub groups of disconnected spots.">
%          <Command>MatlabXT::XTSpotsSplit(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%      <SurpassTab>
%        <SurpassComponent name="bpSpots">
%          <Item name="Split Spots" icon="Matlab" tooltip="Split one spots object in sub groups of disconnected spots.">
%            <Command>MatlabXT::XTSpotsSplit(%i)</Command>
%          </Item>
%        </SurpassComponent>
%      </SurpassTab>
%    </CustomTools>
% 
%
%  Description:
%   
%   Split one spots object in sub groups of disconnected spots.
%   Two spots groups are disconnected if the minimal distance between 
%       their elements is greater than the user-defined threshold.
% 
%

function XTSpotsSplit(aImarisApplicationID, aThreshold)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end


% the user has to create a scene with some spots
vSurpassScene = vImarisApplication.GetSurpassScene;
if isequal(vSurpassScene, [])
    msgbox('Please create some Spots in the Surpass scene!');
    return;
end

% get the spots
vSpots = vImarisApplication.GetFactory.ToSpots(vImarisApplication.GetSurpassSelection);

% search the spots if not previously selected
if ~vImarisApplication.GetFactory.IsSpots(vSpots)        
    for vChildIndex = 1:vSurpassScene.GetNumberOfChildren
        vDataItem = vSurpassScene.GetChild(vChildIndex - 1);
        if isequal(vSpots, [])
            if vImarisApplication.GetFactory.IsSpots(vDataItem)
                vSpots = vImarisApplication.GetFactory.ToSpots(vDataItem);
            end
        end
    end
    % did we find the spots?
    if isequal(vSpots, [])
        msgbox('Please create some spots!');
        return;
    end
end

% get the spots coordinates
vSpotsXYZ = vSpots.GetPositionsXYZ;
vSpotsTime = vSpots.GetIndicesT;
vSpotsRadii = vSpots.GetRadiiXYZ;
vNumberOfSpots = size(vSpotsXYZ, 1);

vSpotsName = vSpots.GetName;
vSpots.SetVisible(false);

% ask for threshold
if nargin<2
    vWarning = '';
    if ~isempty(max(vSpotsTime)) && max(vSpotsTime) > min(vSpotsTime)
        vWarning = ['Warning: Spots at different time points might interfere ', ...
            'and lead to unexpected results. \n'];
    end
    vQuestion = {sprintf(['Split the spots in sub groups that have distance > threshold. \n', ...
        vWarning, ...
        'Please enter the threshold value:']) ...
        };
    vAnswer = inputdlg(vQuestion,'Split spots',1,{'0'});
    if isempty(vAnswer), return; end
    vThreshold = str2double(char(vAnswer));
else
    vThreshold = aThreshold;
end

vProgressDisplay = waitbar(0,'Splitting spots');

% create new group
vSpotsGroup = vImarisApplication.GetFactory.CreateDataContainer;
vSpotsGroup.SetName(sprintf('%s elements [%.2f]', char(vSpotsName), vThreshold));

if vThreshold == 0
    for vSpot = 1:vNumberOfSpots
        vNewSpots = vImarisApplication.GetFactory.CreateSpots;

        vNewSpots.Set(vSpotsXYZ(vSpot,:), vSpotsTime(vSpot), 0);
		vNewSpots.SetRadiiXYZ(vSpotsRadii(vSpot,:));
        
        vNewSpots.SetName(sprintf('%s [%i]', char(vSpotsName), vSpot));

        vNewSpots.SetColorRGBA(floor((256^3)*rand));
        vSpotsGroup.AddChild(vNewSpots, -1);
        
        waitbar(vSpot/vNumberOfSpots)
    end
    
    vSurpassScene.AddChild(vSpotsGroup, -1);
    close(vProgressDisplay);
    return;
end

% build the neighbor matrix
vSpotsXYZT = vSpotsXYZ';
vNeighborMatrix = false(vNumberOfSpots, vNumberOfSpots);

for vSpotsIndex = 1:vNumberOfSpots
    vXYZ = diag(vSpotsXYZ(vSpotsIndex,:))*ones(3,vNumberOfSpots);
    vDistanceList = sqrt(sum( (vXYZ - vSpotsXYZT).^2, 1 )) <= vThreshold;
    vNeighbors = vDistanceList | vNeighborMatrix(vSpotsIndex,:);
    vNeighborMatrix(vNeighbors,vNeighbors) = 1;
    
    waitbar(0.3*vSpotsIndex/vNumberOfSpots)
end

vNumberOfGroups = 0;
vSpots = 1:vNumberOfSpots;
while ~isempty(vSpots)
    vSpotsIndex = vSpots(1);

    vNeighbors = vNeighborMatrix(vSpotsIndex,vSpots);
    vGroupIndices = vSpots(vNeighbors);
    
    vNewSpots = vImarisApplication.GetFactory.CreateSpots;

    vNewSpots.Set(vSpotsXYZ(vGroupIndices,:), vSpotsTime(vGroupIndices), ...
        zeros(sum(vGroupIndices~=0),1));
	vNewSpots.SetRadiiXYZ(vSpotsRadii(vGroupIndices,:));
    
    vNumberOfGroups = vNumberOfGroups + 1;
    vNewSpots.SetName(sprintf('%s [%i]{%i}', char(vSpotsName), vNumberOfGroups, ...
        length(vGroupIndices)));
    
    vNewSpots.SetColorRGBA(floor((256^3)*rand));
    vSpotsGroup.AddChild(vNewSpots, -1);
    
    vSpotsRemaining = vNeighborMatrix(vSpotsIndex, vSpots) == 0;
    vSpots = vSpots(vSpotsRemaining);
    
    waitbar(0.3+0.7*(1-length(vSpots)/vNumberOfSpots))
end

vSurpassScene.AddChild(vSpotsGroup, -1);
close(vProgressDisplay);

